@@ -13,6 +13,11 @@ from collections import defaultdict
 from datetime import datetime
 
 from pyxform import constants
+from pyxform.utils import (
+    BRACKETED_TAG_REGEX,
+    LAST_SAVED_REGEX,
+    LAST_SAVED_INSTANCE_NAME,
+)
 from pyxform.errors import PyXFormError, ValidationError
 from pyxform.external_instance import ExternalInstance
 from pyxform.instance import SurveyInstance
@@ -35,10 +40,6 @@ try:
 except ImportError:
     from functools32 import lru_cache
 
-LAST_SAVED_INSTANCE_NAME = "__last-saved"
-BRACKETED_TAG_REGEX = re.compile(r"\${(last-saved#)?(.*?)}")
-LAST_SAVED_REGEX = re.compile(r"\${last-saved#(.*?)}")
-
 
 def register_nsmap():
     """Function to register NSMAP namespaces with ETree"""
@@ -67,7 +68,7 @@ def is_parent_a_repeat(survey, xpath):
 
 
 @lru_cache(maxsize=None)
-def share_same_repeat_parent(survey, xpath, context_xpath):
+def share_same_repeat_parent(survey, xpath, context_xpath, reference_parent=False):
     """
     Returns a tuple of the number of steps from the context xpath to the shared
     repeat parent and the xpath to the target xpath from the shared repeat
@@ -79,14 +80,20 @@ def share_same_repeat_parent(survey, xpath, context_xpath):
 
         returns (2, '/group_a/name')'
     """
-    context_parent = is_parent_a_repeat(survey, context_xpath)
-    xpath_parent = is_parent_a_repeat(survey, xpath)
-    if context_parent and xpath_parent and xpath_parent in context_parent:
-        context_parts = context_xpath[len(xpath_parent) + 1 :].split("/")
+
+    def _get_steps_and_target_xpath(context_parent, xpath_parent, include_parent=False):
         parts = []
         steps = 1
-        remainder_xpath = xpath[len(xpath_parent) :]
-        xpath_parts = xpath[len(xpath_parent) + 1 :].split("/")
+        if not include_parent:
+            remainder_xpath = xpath[len(xpath_parent) :]
+            context_parts = context_xpath[len(xpath_parent) + 1 :].split("/")
+            xpath_parts = xpath[len(xpath_parent) + 1 :].split("/")
+        else:
+            split_idx = len(xpath_parent.split("/"))
+            context_parts = context_xpath.split("/")[split_idx - 1 :]
+            xpath_parts = xpath.split("/")[split_idx - 1 :]
+            remainder_xpath = "/".join(xpath_parts)
+
         for index, item in enumerate(context_parts[:-1]):
             try:
                 if xpath[len(context_parent) + 1 :].split("/")[index] != item:
@@ -99,9 +106,36 @@ def share_same_repeat_parent(survey, xpath, context_xpath):
                 steps = len(context_parts[index - 1 :])
                 parts = xpath_parts[index - 1 :]
                 break
-
         return (steps, "/" + "/".join(parts) if parts else remainder_xpath)
 
+    context_parent = is_parent_a_repeat(survey, context_xpath)
+    xpath_parent = is_parent_a_repeat(survey, xpath)
+    if context_parent and xpath_parent and xpath_parent in context_parent:
+        include_parent = False
+        if not context_parent == xpath_parent and reference_parent:
+            context_shared_ancestor = is_parent_a_repeat(survey, context_parent)
+            if context_shared_ancestor == xpath_parent:
+                # Check if context_parent is a child repeat of the xpath_parent
+                # If the context_parent is a child of the xpath_parent reference the entire
+                # xpath_parent in the generated nodeset
+                include_parent = True
+                context_parent = context_shared_ancestor
+        return _get_steps_and_target_xpath(context_parent, xpath_parent, include_parent)
+    elif context_parent and xpath_parent:
+        # Check if context_parent and xpath_parent share a common
+        # repeat ancestor
+        context_shared_ancestor = is_parent_a_repeat(survey, context_parent)
+        xpath_shared_ancestor = is_parent_a_repeat(survey, xpath_parent)
+
+        if (
+            xpath_shared_ancestor
+            and context_shared_ancestor
+            and xpath_shared_ancestor == context_shared_ancestor
+        ):
+            return _get_steps_and_target_xpath(
+                context_shared_ancestor, xpath_shared_ancestor
+            )
+
     return (None, None)
 
 
@@ -836,13 +870,98 @@ class Survey(Section):
                 else:
                     self._xpath[element.name] = element.get_xpath()
 
-    def _var_repl_function(self, matchobj, context, use_current=False):
+    def _var_repl_function(
+        self, matchobj, context, use_current=False, reference_parent=False
+    ):
         """
         Given a dictionary of xpaths, return a function we can use to
         replace ${varname} with the xpath to varname.
         """
+
         name = matchobj.group(2)
         last_saved = matchobj.group(1) is not None
+        is_indexed_repeat = matchobj.string.find("indexed-repeat(") > -1
+        indexed_repeat_regex = re.compile(r"indexed-repeat\([^)]+\)")
+        function_args_regex = re.compile(r"\b[^()]+\((.*)\)$")
+
+        def _relative_path(name):
+            """Given name in ${name}, return relative xpath to ${name}."""
+            return_path = None
+            xpath, context_xpath = self._xpath[name], context.get_xpath()
+            # share same root i.e repeat_a from /data/repeat_a/...
+            if (
+                len(context_xpath.split("/")) > 2
+                and xpath.split("/")[2] == context_xpath.split("/")[2]
+            ):
+                # if context xpath and target xpath fall under the same
+                # repeat use relative xpath referencing.
+                steps, ref_path = share_same_repeat_parent(
+                    self, xpath, context_xpath, reference_parent
+                )
+                if steps:
+                    ref_path = ref_path if ref_path.endswith(name) else "/%s" % name
+                    prefix = " current()/" if use_current else " "
+
+                    return_path = prefix + "/".join([".."] * steps) + ref_path + " "
+
+            return return_path
+
+        def _is_return_relative_path():
+            """Determine condition to return relative xpath of current ${name}."""
+            indexed_repeat_relative_path_args_index = [0, 1, 3, 5]
+            current_matchobj = matchobj
+
+            if not last_saved and context:
+                if context["type"] == "text":
+
+                    if not is_indexed_repeat:
+                        return True
+
+                    # It is possible to have multiple indexed-repeat in an expression
+                    indexed_repeats_iter = indexed_repeat_regex.finditer(
+                        matchobj.string
+                    )
+                    for indexed_repeat in indexed_repeats_iter:
+
+                        # Make sure current ${name} is in the correct indexed-repeat
+                        if current_matchobj.end() > indexed_repeat.end():
+                            try:
+                                next(indexed_repeats_iter)
+                                continue
+                            except StopIteration:
+                                return True
+
+                        # ${name} outside of indexed-repeat always using relative path
+                        if (
+                            current_matchobj.end() < indexed_repeat.start()
+                            or current_matchobj.start() > indexed_repeat.end()
+                        ):
+                            return True
+
+                        indexed_repeat_name_index = None
+                        indexed_repeat_args = (
+                            function_args_regex.match(indexed_repeat.group())
+                            .group(1)
+                            .split(",")
+                        )
+                        name_arg = "${{{0}}}".format(name)
+                        for idx, arg in enumerate(indexed_repeat_args):
+                            if name_arg in arg.strip():
+                                indexed_repeat_name_index = idx
+
+                        return (
+                            indexed_repeat_name_index is not None
+                            and indexed_repeat_name_index
+                            not in indexed_repeat_relative_path_args_index
+                        )
+                else:
+                    return not (
+                        context["type"] == "calculate"
+                        and "indexed-repeat" in context["bind"]["calculate"]
+                    )
+
+            return False
+
         intro = (
             "There has been a problem trying to replace %s with the "
             "XPath to the survey element named '%s'." % (matchobj.group(0), name)
@@ -853,41 +972,26 @@ class Survey(Section):
             raise PyXFormError(
                 intro + " There are multiple survey elements" " with this name."
             )
-        if (
-            not last_saved
-            and context
-            and not (
-                context["type"] == "calculate"
-                and "indexed-repeat" in context["bind"]["calculate"]
-            )
-        ):
-            xpath, context_xpath = self._xpath[name], context.get_xpath()
-            # share same root i.e repeat_a from /data/repeat_a/...
-            if (
-                len(context_xpath.split("/")) > 2
-                and xpath.split("/")[2] == context_xpath.split("/")[2]
-            ):
-                # if context xpath and target xpath fall under the same
-                # repeat use relative xpath referencing.
-                steps, ref_path = share_same_repeat_parent(self, xpath, context_xpath)
-                if steps:
-                    ref_path = ref_path if ref_path.endswith(name) else "/%s" % name
-                    prefix = " current()/" if use_current else " "
 
-                    return prefix + "/".join([".."] * steps) + ref_path + " "
+        if _is_return_relative_path():
+            relative_path = _relative_path(name)
+            if relative_path:
+                return relative_path
 
         last_saved_prefix = (
             "instance('" + LAST_SAVED_INSTANCE_NAME + "')" if last_saved else ""
         )
         return " " + last_saved_prefix + self._xpath[name] + " "
 
-    def insert_xpaths(self, text, context, use_current=False):
+    def insert_xpaths(self, text, context, use_current=False, reference_parent=False):
         """
         Replace all instances of ${var} with the xpath to var.
         """
 
         def _var_repl_function(matchobj):
-            return self._var_repl_function(matchobj, context, use_current)
+            return self._var_repl_function(
+                matchobj, context, use_current, reference_parent
+            )
 
         return re.sub(BRACKETED_TAG_REGEX, _var_repl_function, unicode(text))
 
